using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Runtime.InteropServices;

namespace Sanford.Multimedia.Wave
{
    public sealed class WaveformBuffer : IDisposable
    {
        public const int DefaultLength = 4096;

        private const int WHDR_DONE = 0x00000001;

        private WaveHeader header = new WaveHeader();

        private IntPtr headerPtr;

        private byte[] buffer;

        private IntPtr bufferPtr;

        public WaveformBuffer()
        {
            Initialize(DefaultLength);            
        }

        public WaveformBuffer(int length)
        {
            Initialize(length);
        }

        private void Initialize(int length)
        {
            #region Require

            if(length < 0)
            {
                throw new ArgumentOutOfRangeException("length");
            }

            #endregion

            buffer = new byte[length];

            headerPtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(WaveHeader)));

            try
            {
                bufferPtr = Marshal.AllocHGlobal(buffer.Length);
            }
            catch
            {
                Marshal.FreeHGlobal(headerPtr);

                throw;
            }

            header.bufferLength = buffer.Length;
            header.bytesRecorded = 0;
            header.flags = 0;
            header.data = bufferPtr;

            Marshal.StructureToPtr(header, headerPtr, true);
        }

        public void Clear()
        {
            Array.Clear(buffer, 0, buffer.Length);
        }

        public byte[] GetBuffer()
        {
            return buffer;
        }

        internal void CopyToMemory()
        {
            Marshal.Copy(buffer, 0, bufferPtr, buffer.Length);
        }

        public byte this[int index]
        {
            get
            {
                return buffer[index];
            }
            set
            {
                buffer[index] = value;
            }
        }

        internal bool IsDone
        {
            get
            {
                header = (WaveHeader)Marshal.PtrToStructure(headerPtr, typeof(WaveHeader));

                return (header.flags & WHDR_DONE) == WHDR_DONE;
            }
        }

        public int Length
        {
            get
            {
                return buffer.Length;
            }
        }

        internal IntPtr HeaderPointer
        {
            get
            {
                return headerPtr;
            }
        }

        #region IDisposable Members

        public void Dispose()
        {            
            Marshal.FreeHGlobal(headerPtr);
            Marshal.FreeHGlobal(bufferPtr);
        }

        #endregion
    }
}
