using System;
using System.Diagnostics;
using System.IO;
using System.Threading;
using Sanford.Multimedia.Wave;
using Sanford.Multimedia.Timers;

namespace Tester
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                OutputDevice device = new OutputDevice(0, 2, 44100, 16);
                WaveformBuffer[] waves = new WaveformBuffer[1];
                IAsyncResult[] results = new IAsyncResult[waves.Length];
                object lockObject = new object();

                for(int i = 0; i < waves.Length; i++)
                {
                    waves[0] = SynthesizeSine(1800);
                }

                Time t = device.GetPosition(TimeType.Samples);

                Console.WriteLine(t.samples.ToString());

                device.Send(waves[0]);

                t = device.GetPosition(TimeType.Samples);

                Console.WriteLine(t.samples.ToString());

                device.Close();

                for(int i = 0; i < waves.Length; i++)
                {
                    waves[i].Dispose();
                }
            }
            catch(Exception ex)
            {
                Console.WriteLine(ex.Message);
            }

            Console.WriteLine("Done.");
            Console.Read();
        }

        static WaveformBuffer SynthesizeSine(int length)
        {
            WaveformBuffer wave = new WaveformBuffer(length * 4);
            float increment = 98.0f / 44100;
            float accumulator = 0;
            float result;
            short sample;
            int n = 0;

            for(int i = 0; i < length; i++)
            {
                result = (float)Math.Sin(Math.PI * 2 * accumulator);
                sample = (short)(short.MaxValue * result);
                wave[n] = (byte)sample;
                n++;
                wave[n] = (byte)(sample >> 8);
                n++;
                wave[n] = (byte)sample;
                n++;
                wave[n] = (byte)(sample >> 8);
                n++;

                accumulator += increment;

                if(accumulator >= 1)
                {
                    accumulator -= 1;
                }
            }

            return wave;
        }
    }
}
